/*
 * output_helper.c
 * CS3243 Homework 2I: Hashiwokakero
 *
 * Sample C code framework for an output helper for Hashiwokakero
 */

#include <stdio.h>
#include <stdlib.h>

/*
 * Global variables.
 */

int g_numrows;
int g_numcols;
char** g_grid;

/*
 * Reads input file subject to specifications on: 
 * http://www.comp.nus.edu.sg/~kanmy/courses/3243_2010/hw-hashiwokakero.html
 */

void readInputFile(char* inputfile) {
  FILE* fp;
  char* buf;
  int r;
  int c;
  
  /* Open input file. */
  fp = fopen(inputfile, "r");
  if (fp == NULL) {
    fprintf(stderr, "Error: Unable to open input file %s\n", inputfile);
    exit(1);
  }
  
  /* Read input file and construct grid. */
  fscanf(fp, "%d %d", &g_numrows, &g_numcols);
  g_grid = (char**)malloc(sizeof(char*) * g_numrows);
  for (r = 0; r < g_numrows; r++)
    g_grid[r] = (char*)malloc(sizeof(char) * g_numcols);
  buf = (char*)malloc((g_numcols + 1) * sizeof(char));
  for (r = 0; r < g_numrows; r++) {
    fscanf(fp, "%s", buf);
    for (c = 0; c < g_numcols; c++)
      g_grid[r][c] = buf[c];
  }
  
  /* Close file. */
  fclose(fp);
}

/*
 * Reads the solution file generated by the Prolog run.
 *
 * Format is x1,y1 of island 1; x2,y2 of island 2, and number of
 * bridges between them.
 *
 */

void readPrologFile(char* prologfile) {
  FILE* fp;
  int srcrow;
  int srccol;
  int tgtrow;
  int tgtcol;
  int type;
  
  /* Open Prolog file. */
  fp = fopen(prologfile, "r");
  if (fp == NULL) {
    fprintf(stderr, "Error: Unable to open Prolog solution file %s\n", prologfile);
    exit(1);
  }
  
  /* Read input file and construct grid. */
  while (fscanf(fp, "%d %d %d %d %d\n", &srcrow, &srccol, &tgtrow, &tgtcol, &type) == 5) { 
    int i;
    if (type == 0) { continue; } /* ignore type 0 (no bridge), if present */
    if (srcrow == tgtrow) {		       /* horizontal bridge */
      if (srccol < tgtcol) {
	for (i = srccol+1; i < tgtcol; i++) 
	  g_grid[srcrow][i] = (type == 1) ? '-' : '='; 
      } else if (srccol > tgtcol) {
	for (i = tgtcol+1; i < srccol; i++) 
	  g_grid[srcrow][i] = (type == 1) ? '-' : '='; 
      } else {
	fprintf (stderr, "Problem with prolog output file\n");
      }
    } else if (srccol == tgtcol) {		 /* vertical bridge */
      if (srcrow < tgtrow) {
	for (i = srcrow+1; i < tgtrow; i++) 
	  g_grid[i][srccol] = (type == 1) ? '|' : '\"'; 
      } else if (srcrow > tgtrow) {
	for (i = tgtrow+1; i < srcrow; i++) 
	  g_grid[i][srccol] = (type == 1) ? '|' : '\"'; 
      } else {
	fprintf (stderr, "Problem with prolog output file\n");
      }
    } else {
      fprintf (stderr, "Problem with prolog output file\n");
    }
  }
  /* Close file. */
  fclose(fp);
}

/*
 * Writes the final output file.
 */

void writeOutputFile(char* outputfile) {
  FILE* fp;
  int r;
  int c;
  
  /* Open output file. */
  fp = fopen(outputfile, "w");
  if (fp == NULL) {
    fprintf(stderr, "Error: Unable to open output file %s\n", outputfile);
    exit(1);
  }
  
  /* Writes the output grid. */
  for (r = 0; r < g_numrows; r++) {
    for (c = 0; c < g_numcols; c++)
      fprintf(fp, "%c", g_grid[r][c]);
    fprintf(fp, "\n");
  }
  
  /* Close file. */
  fclose(fp);
}

/*
 * Usage message, run if invoked incorrectly.
 */

void usage(char* progname) {
  fprintf (stderr, "Usage: %s input_file prolog_file output_file\n", progname);
  exit(1);
}

/*
 * Main function.
 */

int main(int argc, char** argv) {
  /* Check number of parameters. */
  if (argc != 4)
    usage(argv[0]);
  
  /* Read the grid from the input file. */
  readInputFile(argv[1]);
  
  /* Read the solution file generated by the Prolog run. */
  readPrologFile(argv[2]);
  
  /* Write the query file. */
  writeOutputFile(argv[3]);
  
  return 0;
}
